/* Xmmsfind - A small searchplugin for the X Multimedia Player
 * Copyright (C) 2001-2002 Isak Savo <iso01001@student.mdh.se> 
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public Licensse as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This program is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.  You
 * should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <bmp/configfile.h>
#include <bmp/beepctrl.h>
#include "config.h"
#include "globals.h"

_cfg_struct cfg_x;
static GtkWidget *cfg_window;
GtkWidget *cfg_chk_title, *cfg_chk_file, *cfg_chk_enqueue, *cfg_rdo_xmms_enqueue, *cfg_rdo_list_enqueue; 
GtkWidget *cfg_chk_force_rdm_off, *cfg_chk_smart_enqueue;
GtkWidget *cfg_chk_show_title, *cfg_chk_center, *cfg_chk_persistent;

void cfg_write_changes (_cfg_struct cfg_local)
{
    ConfigFile *cfgfile;
    
    if ((cfgfile = xmms_cfg_open_default_file()) != NULL)
    {
	xmms_cfg_write_boolean(cfgfile, "beepfind", "match_title", cfg_local.match_title);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "match_path", cfg_local.match_path);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "enqueue", cfg_local.enqueue);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "xmms_enqueue", cfg_local.xmms_enqueue);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "list_enqueue", cfg_local.list_enqueue);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "force_rdm_off", cfg_local.force_rdm_off);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "smart_enqueue", cfg_local.smart_enqueue);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "show_title", cfg_local.show_title);
	xmms_cfg_write_boolean(cfgfile, "xmmsfind", "center", cfg_local.center);
	xmms_cfg_write_boolean(cfgfile, "beepfind", "persistent", cfg_local.persistent);
	xmms_cfg_write_int(cfgfile, "beepfind", "size_x", cfg_local.size_x);
	xmms_cfg_write_int(cfgfile, "beepfind", "size_y", cfg_local.size_y);
	xmms_cfg_write_default_file (cfgfile);
	xmms_cfg_free (cfgfile);
    }
    else
      g_warning ("xmmsfind: Unable to write changes to xmms config file!\n");
}

void exit_cfg_save_changes (GtkWidget *btn, gpointer data)
{
    
    cfg_x.match_title =   gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_title));
    cfg_x.match_path =    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_file));
    cfg_x.enqueue =       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_enqueue));
    cfg_x.xmms_enqueue =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_rdo_xmms_enqueue));
    cfg_x.list_enqueue =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_rdo_list_enqueue));
    cfg_x.force_rdm_off = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_force_rdm_off));
    cfg_x.smart_enqueue = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_smart_enqueue));
    cfg_x.show_title =    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_show_title));
    cfg_x.center =    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_center));
    cfg_x.persistent =    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_chk_persistent));
    
    cfg_write_changes (cfg_x);
    gtk_widget_destroy (GTK_WIDGET (cfg_window));
    cfg_window = NULL;
}

void exit_cfg_discard_changes (GtkWidget *btn, gpointer data)
{
    gtk_widget_destroy (GTK_WIDGET (cfg_window));
    cfg_window = NULL;
}

void xmmsfind_do_read_config_file (void)
{
    ConfigFile *cfgfile;
    
    cfgfile = xmms_cfg_open_default_file();
    /* split out to test evey read for when options change between version */
    if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "match_title", &(cfg_x.match_title))) 
	    cfg_x.match_title = TRUE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "match_path", &(cfg_x.match_path))) 
	    cfg_x.match_path = FALSE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "enqueue", &(cfg_x.enqueue))) 
        cfg_x.enqueue = FALSE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "xmms_enqueue", &(cfg_x.xmms_enqueue))) 
	    cfg_x.xmms_enqueue = TRUE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "list_enqueue", &(cfg_x.list_enqueue)))
	    cfg_x.list_enqueue = FALSE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "force_rdm_off", &(cfg_x.force_rdm_off)))
        cfg_x.force_rdm_off = TRUE;
	if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "smart_enqueue", &(cfg_x.smart_enqueue))) 
	    cfg_x.smart_enqueue = FALSE;
    if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "show_title", &(cfg_x.show_title))) 
	    cfg_x.show_title = FALSE; 
    if (!xmms_cfg_read_boolean (cfgfile, "xmmsfind", "center", &(cfg_x.center)))
	    cfg_x.center = TRUE;
    if (!xmms_cfg_read_boolean (cfgfile, "beepfind", "persistent", &(cfg_x.persistent))) 
	    cfg_x.persistent = FALSE;
	if (!xmms_cfg_read_int (cfgfile, "beepfind", "size_x", &(cfg_x.size_x))) 
        cfg_x.size_x = 325;
	if (!xmms_cfg_read_int (cfgfile, "beepfind", "size_y", &(cfg_x.size_y))) 
        cfg_x.size_y = 400;
	xmms_cfg_free (cfgfile);
}

void cfg_check_list_enqueue_changed (void)
{
    gtk_widget_set_sensitive (cfg_chk_force_rdm_off, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_rdo_list_enqueue)));
    gtk_widget_set_sensitive (cfg_chk_smart_enqueue, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_rdo_list_enqueue)));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_force_rdm_off), gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cfg_rdo_list_enqueue)));
}

void xmmsfind_config (void)
{
    GtkWidget *main_vbox, *cfg_hbox;
    GtkWidget *cfg_main_vbox;
    GtkWidget *cfg_btn_ok, *cfg_btn_abort, *lbl;
    GtkWidget *hbox, *vbox, *lbl_indent;
    GtkTooltips *tip;
    
    if (cfg_window)
    {
	gtk_widget_grab_focus (cfg_window);
	return;
    }
    
    tip = gtk_tooltips_new ();
    gtk_tooltips_enable (tip);
    
    xmmsfind_do_read_config_file ();
    cfg_window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_type_hint ( GTK_WINDOW (cfg_window), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_container_border_width (GTK_CONTAINER (cfg_window), 6);
    gtk_window_set_position (GTK_WINDOW (cfg_window), GTK_WIN_POS_CENTER);
    gtk_window_set_title (GTK_WINDOW (cfg_window), "Beepfind Configuration");
    g_signal_connect (G_OBJECT (cfg_window), "delete_event",
		      G_CALLBACK (exit_cfg_discard_changes), cfg_window);

    /* VBox used for the entire dialog */
    cfg_main_vbox = gtk_vbox_new (FALSE, 12);
    
    cfg_chk_title = gtk_check_button_new_with_label ("Title/Filename");
    cfg_chk_file = gtk_check_button_new_with_label ("Directory Path");
    cfg_chk_enqueue = gtk_check_button_new_with_label ("Enqueue songs by default");
    cfg_rdo_xmms_enqueue = gtk_radio_button_new_with_label (NULL, "BMP enqueue");
    cfg_rdo_list_enqueue = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (cfg_rdo_xmms_enqueue), "Playlist enqueue");
    cfg_chk_force_rdm_off = gtk_check_button_new_with_label ("Force random off");
    cfg_chk_smart_enqueue = gtk_check_button_new_with_label ("Use smart enqueueing"); /* FIXME: name? */
    cfg_chk_show_title = gtk_check_button_new_with_label ("Get titleinfo from BMP");
    cfg_chk_center = gtk_check_button_new_with_label ("Center the Beepfind window");
    cfg_chk_persistent = gtk_check_button_new_with_label ("Do not close Beepfind window");
    
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_title,
			 "Makes Beepfind match its search with the titlestring and filename",
			 "The titlestring is the one got from BMP (i.e. the one you see in the playlist)");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_file,
			 "Makes Beepfind match its search with the directory path",
			 "The directory is the entire path to the file (i.e. '/usr/mp3/rock/')");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_enqueue,
			 "Will force Beepfind to enqueue the file, rather than jumping to it.",
			 "Enqueuing a file will set it to be played next, rather than immediatly.");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_rdo_xmms_enqueue,
                         "Makes Beepfind use BMP's built-in enqueue system and does not edit the actual playlist.",
			 "This feature allows you to queue songs without editing your playlist and still works with random mode on.");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_rdo_list_enqueue,
			 "Will force Beepfind to move the file to the position after the currently playing, editing the playlist.",
			 "Enqueuing a file will make Beepfind move it to the song position after the one that is currently being played.");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_force_rdm_off,
			 "When enqueueing file, turn off random (shuffle) mode if its on.",
			 "You should probably use this if you want to enqueue by default since enqueueing wont work otherwise");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_smart_enqueue,
			 "Will add an offset to the enqueue so that the next song enqueued will be moved after the previous one",
			 "This is useful if you usually enqueues more than one song at time and you wish for them to appear in the order you enqueued them.");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_show_title,
			 "Tells Beepfind to query BMP for the titlestring. With this option enabled to speed up Beepfind set 'Read info on' to 'load' in the BMP preferences dialog under options.",
			 "If selected, then the string shown in the searchbox will be tha basename of the file");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_center,
			"This will make the Beepfind window pop up in the center of the screen.",
			"Disabling this will allow the window manager to choose a location.");
    gtk_tooltips_set_tip (GTK_TOOLTIPS (tip), cfg_chk_persistent,
			 "This will make the Beepfind window stay open after a song has been changed",
			 "This is useful if you want to use Beepfind as a complement to the ordinary playlist");

    /* Start packing stuff up */
    
    lbl_indent = gtk_label_new ("    "); /* Used for indentation */
    
    main_vbox = gtk_vbox_new (FALSE, 6);
    
    lbl = gtk_label_new ("<b>Match search by</b>");
    gtk_label_set_use_markup (GTK_LABEL (lbl), TRUE);
    gtk_misc_set_alignment (GTK_MISC (lbl), 0, 0.5);

    vbox = gtk_vbox_new (FALSE, 6);
    hbox = gtk_hbox_new (FALSE, 6);

    /* Pack the search-match controls into a hbox */
    gtk_box_pack_start (GTK_BOX (hbox), lbl_indent, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_title, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_file, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (main_vbox), lbl, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);

    lbl_indent = gtk_label_new ("    "); /* Used for indentation */

    lbl = gtk_label_new ("<b>Enqueuing</b>");
    gtk_label_set_use_markup (GTK_LABEL (lbl), TRUE);
    gtk_misc_set_alignment (GTK_MISC (lbl), 0, 0.5);

    vbox = gtk_vbox_new (FALSE, 6);
    hbox = gtk_hbox_new (FALSE, 6);

    gtk_box_pack_start (GTK_BOX (hbox), lbl_indent, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_enqueue, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_rdo_xmms_enqueue, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_rdo_list_enqueue, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_force_rdm_off, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_smart_enqueue, FALSE, FALSE, 0);
    
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (main_vbox), lbl, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);


    /* MISC settings */
    lbl_indent = gtk_label_new ("    "); /* Used for indentation */

    lbl = gtk_label_new ("<b>Settings</b>");
    gtk_label_set_use_markup (GTK_LABEL (lbl), TRUE);
    gtk_misc_set_alignment (GTK_MISC (lbl), 0, 0.5);

    vbox = gtk_vbox_new (FALSE, 6);
    hbox = gtk_hbox_new (FALSE, 6);

    /* Pack the misc controls into a hbox */
    gtk_box_pack_start (GTK_BOX (hbox), lbl_indent, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_show_title, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_persistent, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), cfg_chk_center, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (main_vbox), lbl, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);

    
    /* Set the togglebuttons to the state last saved */
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_title), cfg_x.match_title);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_file), cfg_x.match_path);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_enqueue), cfg_x.enqueue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_rdo_xmms_enqueue), cfg_x.xmms_enqueue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_rdo_list_enqueue), cfg_x.list_enqueue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_force_rdm_off), cfg_x.force_rdm_off);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_smart_enqueue), cfg_x.smart_enqueue);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_show_title), cfg_x.show_title);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_center), cfg_x.center);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cfg_chk_persistent), cfg_x.persistent);

    g_signal_connect (G_OBJECT (cfg_rdo_xmms_enqueue), "clicked",
		      G_CALLBACK (cfg_check_list_enqueue_changed), NULL);
    g_signal_connect (G_OBJECT (cfg_rdo_list_enqueue), "clicked",
		      G_CALLBACK (cfg_check_list_enqueue_changed), NULL);
    
    gtk_widget_set_sensitive (cfg_chk_force_rdm_off, cfg_x.list_enqueue);	/* Only clickable if cfg_chk_list_enqueue is toggled */
    gtk_widget_set_sensitive (cfg_chk_smart_enqueue, cfg_x.list_enqueue);
    
    cfg_btn_ok = gtk_button_new_from_stock(GTK_STOCK_OK); 
    cfg_btn_abort = gtk_button_new_from_stock(GTK_STOCK_CANCEL);

    /* Use g_signal_connect instead of deprecated gtk_signal_connect() */
    g_signal_connect (G_OBJECT (cfg_btn_ok), "clicked",
		      G_CALLBACK (exit_cfg_save_changes), cfg_window);
    g_signal_connect (G_OBJECT (cfg_btn_abort), "clicked",
		      G_CALLBACK (exit_cfg_discard_changes), cfg_window);
   
    GTK_WIDGET_SET_FLAGS(cfg_btn_abort, GTK_CAN_DEFAULT);
    GTK_WIDGET_SET_FLAGS(cfg_btn_ok, GTK_CAN_DEFAULT);
    
    cfg_hbox = gtk_hbutton_box_new(); 
    gtk_button_box_set_layout(GTK_BUTTON_BOX(cfg_hbox), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(cfg_hbox), 5);

    gtk_box_pack_start (GTK_BOX (cfg_hbox), cfg_btn_abort, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (cfg_hbox), cfg_btn_ok, FALSE, FALSE, 0);
    
    gtk_box_pack_start (GTK_BOX (cfg_main_vbox), main_vbox, TRUE, TRUE, 6); 
    gtk_box_pack_start (GTK_BOX (cfg_main_vbox), cfg_hbox, FALSE, FALSE, 0); 
    
    gtk_container_add (GTK_CONTAINER (cfg_window), cfg_main_vbox);
    
    gtk_widget_show_all (cfg_window);
    gtk_widget_grab_default(cfg_btn_ok);
}

